/* Copyright 2016-2017 Research Foundation State University of New York   */

/* This file is part of QUB Online.                                       */

/* QUB Online is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Online is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Online program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

#include <vector>
using std::vector;

#include "qubx_amphist.h"

class AmpHistMaker {
public:
  int Nbin;
  int total;
  float lo, hi;
  int Nscan;
  float binWidth;
  vector<float> bins;
  vector<int> counts;

  AmpHistMaker(int Nbin_)
    : Nbin(Nbin_), total(0), Nscan(0), bins(Nbin_), counts(Nbin_)
  {}
  void reset(bool countsOnly) {
    if ( ! countsOnly ) {
      Nscan = 0;
    }
    total = 0;
    for (int i=0; i<Nbin; ++i) {
      counts[i] = 0;
    }
  }
  void setNbin(int Nbin_) {
    Nbin = Nbin_;
    bins.resize(Nbin_);
    counts.resize(Nbin_);
    reset(false);
  }
  void scan(float *data, int N) {
    int i = 0;
    if ( (! Nscan) && N ) {
      lo = hi = data[0];
      i = 1;
    }
    for (; i<N; ++i) {
      float d = data[i];
      if ( d < lo ) {
        lo = d;
      }
      if ( d > hi ) {
        hi = d;
      }
    }
    Nscan += N;
  }
  void bin(float *binsOut) {
    float one = (hi - lo) / ((float) Nbin);
    float b = lo + 0.5f * one;
    for (int i=0; i<Nbin; ++i, b += one) {
      bins[i] = binsOut[i] = b;
    }
    binWidth = one;
  }
  void add(float *data, int N) {
    float offset = -lo;
    float scale = 1.0f / (float) binWidth;
    for (int i=0; i<N; ++i, ++data) {
      int b = (int) ((*data + offset) * scale);
      if ( b == Nbin )
        --b;
      ++counts[b];
    }
    total += N;
  }
  void hist(float *histOut) {
    double one = 1.0 / total;
    for (int i=0; i<Nbin; ++i)
      histOut[i] = (float) (counts[i] * one);
  }
};

extern "C" void* QUBFAST_API qubx_amphist_init(int Nbin) {
  return new AmpHistMaker(Nbin);
}

extern "C" void QUBFAST_API qubx_amphist_free(void *obj) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  delete hm;
}

extern "C" void QUBFAST_API qubx_amphist_reset(void *obj, int countsOnly) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  hm->reset(countsOnly);
}

extern "C" void QUBFAST_API qubx_amphist_setNbin(void *obj, int Nbin) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  hm->setNbin(Nbin);
}

extern "C" void QUBFAST_API qubx_amphist_scan(void *obj, float *data, int N) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  hm->scan(data, N);
}

extern "C" void QUBFAST_API qubx_amphist_bin(void *obj, float *bins) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  hm->bin(bins);
}

extern "C" void QUBFAST_API qubx_amphist_add(void *obj, float *data, int N) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  hm->add(data, N);
}

extern "C" void QUBFAST_API qubx_amphist_hist(void *obj, float *hist) {
  AmpHistMaker *hm = (AmpHistMaker *) obj;
  hm->hist(hist);
}

